package eu.jonahbauer.chat.bot.database.impl;

import eu.jonahbauer.chat.bot.database.Entity;
import org.jetbrains.annotations.NotNull;

import java.sql.Connection;
import java.util.stream.Collectors;

final class UpdaterFactory extends AbstractFactory<@NotNull Updater<?>> {
    public static final @NotNull UpdaterFactory INSTANCE = new UpdaterFactory();

    private UpdaterFactory() {}

    @Override
    protected <T extends Record> @NotNull Updater<?> computeValue0(@NotNull Class<T> type) {
        if (!Entity.class.isAssignableFrom(type)) throw new IllegalArgumentException();
        return bridge(type);
    }

    @SuppressWarnings("unchecked")
    private <T extends Record & Entity> @NotNull Updater<T> bridge(@NotNull Class<?> type) {
        return computeValue1((Class<T>) type);
    }

    private <T extends Record & Entity> @NotNull Updater<T> computeValue1(@NotNull Class<T> type) {
        var table = Helper.getQuotedTableName(type);
        var preparer = Preparer.get(type);
        var columns = preparer.columns();

        var query = STR."""
                    UPDATE \{table}
                    SET \{columns.stream().map(col -> col + " = ?").collect(Collectors.joining(", "))}
                    WHERE id = ?
                    """;

        return (Connection connection, T entity) -> {
            Helper.logSqlStatement(query);
            try (var stmt = connection.prepareStatement(query)) {
                preparer.prepare(stmt, entity);
                stmt.setLong(columns.size() + 1, entity.id());
                return stmt.executeUpdate() == 1;
            }
        };
    }
}
