package eu.jonahbauer.chat.bot.pizza.model;

import org.jetbrains.annotations.NotNull;

import static eu.jonahbauer.chat.bot.pizza.util.FormatUtils.tabular;

public record Receipt(double total, double tipped, double pending, double pendingTipped) {
    public Receipt(double total, boolean payed, double tip) {
        this(total, total * tip, payed ? 0 : total, payed ? 0 : total * tip);
    }

    public boolean payed() {
        return pending == 0;
    }

    public boolean unpayed() {
        return total == pending;
    }

    public @NotNull String totalAsString() {
        return tabular(total, 0) + " €" + (tipped != 0 ? " (" + tabular(tipped, 0) + " €)" : "");
    }

    public @NotNull String pendingAsString() {
        return tabular(pending, 0) + " €" + (pendingTipped != 0 ? " (" + tabular(pendingTipped, 0) + " €)" : "");
    }

    public static @NotNull Receipt empty() {
        return new Receipt(0, 0, 0, 0);
    }

    public static @NotNull Receipt sum(@NotNull Receipt first, @NotNull Receipt second) {
        return new Receipt(
                first.total + second.total,
                first.tipped + second.tipped,
                first.pending + second.pending,
                first.pendingTipped + second.pendingTipped
        );
    }
}
