package eu.jonahbauer.chat.bot.pizza.model.view;

import eu.jonahbauer.chat.bot.pizza.model.Receipt;
import eu.jonahbauer.chat.bot.pizza.model.table.Order;
import eu.jonahbauer.chat.bot.pizza.model.table.Pizza;
import eu.jonahbauer.chat.bot.pizza.model.table.User;
import lombok.experimental.FieldNameConstants;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.time.Instant;
import java.util.Locale;

import static eu.jonahbauer.chat.bot.pizza.util.FormatUtils.strikethrough;
import static eu.jonahbauer.chat.bot.pizza.util.FormatUtils.tabular;

@FieldNameConstants
public record OrderItemDetail(
        // OrderItem
        long id,
        @Nullable String notes,
        // Order
        long orderId,
        @NotNull String orderName,
        @NotNull Instant orderDeadline,
        @Nullable Double orderPayment,
        // Event
        long eventId,
        // Restaurant
        long restaurantId,
        @NotNull String restaurantName,
        // Pizza
        long pizzaId,
        int pizzaNumber,
        @NotNull String pizzaName,
        double pizzaPrice,
        // User
        long userId,
        @NotNull String userName,
        long userParts,
        boolean userPayed,
        long userCount
) {
    public @NotNull Order order() {
        return new Order(orderId, eventId, restaurantId, orderName, orderDeadline, orderPayment);
    }

    public @NotNull Pizza pizza() {
        return new Pizza(pizzaId, restaurantId, pizzaNumber, pizzaName, "N/A", pizzaPrice);
    }

    public @NotNull User user() {
        return new User(userId, userName);
    }

    public @NotNull Receipt receipt(double tip) {
        return new Receipt(userPrice(), userPayed(), tip);
    }

    public double userPrice() {
        return userParts() * pizzaPrice() / userCount();
    }

    public @NotNull String toPrettyString() {
        String price;
        if (userCount() == 1) {
            price = tabular(pizzaPrice(), 0) + " €";
        } else {
            price = tabular(pizzaPrice(), 0) + " € / " + tabular(userCount(), 0) + " = " + tabular(userPrice(), 0) + " €";
        }
        if (userPayed()) price = strikethrough(price);

        if (notes() != null) {
            return STR."\{id()} | \{pizzaName().toUpperCase(Locale.ROOT)} \{notes()} | \{price}";
        } else {
            return STR."\{id()} | \{pizzaName().toUpperCase(Locale.ROOT)} | \{price}";
        }
    }
}
