package eu.jonahbauer.chat.bot.pizza.util;

import lombok.experimental.UtilityClass;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.TestOnly;

import java.util.List;
import java.util.Locale;
import java.util.function.Function;
import java.util.stream.Collectors;

@UtilityClass
public class FormatUtils {
    public static @NotNull String tabular(long number) {
        return tabular(number, 2);
    }

    public static @NotNull String tabular(long number, int width) {
        return tabular(String.format(Locale.ROOT, "%" + (width == 0 ? "" : width) + "d", number));
    }

    public static @NotNull String tabular(double number) {
        return tabular(number, 6);
    }

    public static @NotNull String tabular(double number, int width) {
        return tabular(String.format(Locale.ROOT, "%" + (width == 0 ? "" : width) + ".2f", number));
    }

    private static @NotNull String tabular(@NotNull String string) {
        var out = new StringBuilder();
        for (int i = 0, length = string.length(); i < length; i++) {
            var chr = string.charAt(i);
            if (chr == ' ') {
                out.append('\u2007'); // figure space
            } else {
                out.append('\u2060'); // word-joiner to prevent kerning
                out.append(chr);
            }
        }
        return out.toString();
    }

    @SuppressWarnings("UnnecessaryUnicodeEscape")
    public static @NotNull String strikethrough(@NotNull String string) {
        var out = new StringBuilder();
        for (int i = 0, length = string.length(); i < length; i++) {
            out.append('\u0336').append(string.charAt(i));
        }
        out.append('\u0335');
        return out.toString();
    }

    @TestOnly
    @SuppressWarnings("UnnecessaryUnicodeEscape")
    public static @NotNull String strip(@NotNull String string) {
        var out = new StringBuilder();
        for (int i = 0, length = string.length(); i < length; i++) {
            var chr = string.charAt(i);
            if (chr != '\u0336' && chr != '\u0335' && chr != '\u2007' && chr != '\u2060') {
                out.append(chr);
            }
        }
        return out.toString();
    }

    public static @NotNull String join(@NotNull List<@NotNull String> strings) {
        return switch (strings.size()) {
            case 0 -> "";
            case 1 -> strings.getFirst();
            default -> String.join(", ", strings.subList(0, strings.size() - 1)) + " und " + strings.getLast();
        };
    }

    public static <T> @NotNull String join(@NotNull List<@NotNull T> items, @NotNull Function<? super T, String> toString) {
        return join(items.stream().map(toString).toList());
    }

    public static <T> @NotNull String join(@NotNull String separator, @NotNull List<@NotNull T> items, @NotNull Function<? super T, String> toString) {
        return items.stream().map(toString).collect(Collectors.joining(separator));
    }
}
