package eu.jonahbauer.chat.server.management;

import eu.jonahbauer.chat.server.management.annotations.ManagedAttribute;
import eu.jonahbauer.chat.server.management.annotations.ManagedOperation;
import lombok.SneakyThrows;
import org.jetbrains.annotations.NotNull;

import javax.management.MXBean;
import javax.management.ObjectName;
import java.util.List;

@SuppressWarnings("unused")
@MXBean
public interface ChatBotSupportMXBean {

    @SneakyThrows
    static @NotNull ObjectName getObjectName(@NotNull String name) {
        return ObjectName.getInstance(STR."eu.jonahbauer.chat.bot.server:component=ChatBots,name=\{quote(name)}");
    }

    private static @NotNull String quote(@NotNull String name) {
        var needsQuoting = name.chars().anyMatch(chr -> chr == ',' || chr == '=' || chr == ':' || chr == '"' || chr == '*' || chr == '?');
        return needsQuoting ? ObjectName.quote(name) : name;
    }

    @ManagedAttribute(description = "the bot's name")
    @NotNull String getName();

    @ManagedAttribute(description = "the bot's type")
    default @NotNull String getType() {
        return getConfig().getType();
    }

    @ManagedAttribute(description = "the channels the bot is active in")
    default @NotNull List<@NotNull String> getChannels() {
        return getConfig().unwrap().getChannels();
    }

    @ManagedAttribute(description = "whether the bot sets the bottag for its messages")
    default boolean isBotTag() {
        return getConfig().unwrap().isBotTag();
    }

    @ManagedAttribute(description = "whether the bot reacts to messages from other bots")
    default boolean isIgnoreBots() {
        return getConfig().unwrap().isIgnoreBots();
    }

    @ManagedAttribute(description = "whether the bot sends messages with public id")
    default boolean isPublicId() {
        return getConfig().unwrap().isPublicId();
    }

    @ManagedAttribute(description = "the bot's config")
    @NotNull BotConfigSupport getConfig();

    @ManagedOperation(description = "stops this bot", impact = ManagedOperation.Impact.ACTION)
    void stop() throws InterruptedException;
}
