package eu.jonahbauer.chat.server.management;

import eu.jonahbauer.chat.server.management.annotations.ManagedAttribute;
import eu.jonahbauer.chat.server.management.annotations.ManagedOperation;
import eu.jonahbauer.chat.server.management.annotations.ManagedParameter;
import lombok.SneakyThrows;
import org.jetbrains.annotations.NotNull;

import javax.management.MXBean;
import javax.management.ObjectName;
import java.util.Date;

@SuppressWarnings("unused")
@MXBean
public interface SocketSupportMXBean {

    @SneakyThrows
    static @NotNull ObjectName getObjectName(@NotNull String channel) {
        return ObjectName.getInstance(STR."eu.jonahbauer.chat.bot.server:component=Sockets,name=\{quote(channel)}");
    }

    private static @NotNull String quote(@NotNull String channel) {
        var needsQuoting = channel.chars().anyMatch(chr -> chr == ',' || chr == '=' || chr == ':' || chr == '"' || chr == '*' || chr == '?');
        return needsQuoting ? ObjectName.quote(channel) : channel;
    }

    @ManagedAttribute(description = "the socket's state")
    SocketState getState();

    @ManagedAttribute(description = "the channel this socket is connected to")
    String getChannel();

    @ManagedAttribute(description = "the time when the current cooldown will end")
    Date getCooldownUntil();


    @ManagedOperation(description = "stops this socket", impact = ManagedOperation.Impact.ACTION)
    void stop() throws InterruptedException;

    @ManagedOperation(description = "forcefully restarts the socket when its currently on cooldown", impact = ManagedOperation.Impact.ACTION)
    void restart();

    @ManagedOperation(description = "sends a message via this socket", impact = ManagedOperation.Impact.ACTION)
    default void send(
            @ManagedParameter(name = "name") String name,
            @ManagedParameter(name = "message") String message
    ) {
        send(name, message, true, true);
    }

    @ManagedOperation(description = "sends a message via this socket", impact = ManagedOperation.Impact.ACTION)
    void send(
            @ManagedParameter(name = "name") String name,
            @ManagedParameter(name = "message") String message,
            @ManagedParameter(name = "bottag") boolean bottag,
            @ManagedParameter(name = "publicic") boolean publicid
    );

}
